function ObjectionStartup() {}

ObjectionStartup.prototype = {
	classID: Components.ID("{10060352-006A-11DD-84D9-524756D89593}"),
	contractID: "@objection/startup;1",
	classDescription: "Objection Startup",

	// get a reference to the prompt service component. For debugging.
	promptService: Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),

	nsILocalFile: new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile"),
	preferences: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),

	debugLevel: 10,

	QueryInterface: function(aIID)
	{
		if(!aIID.equals(CI.nsISupports) && !aIID.equals(CI.nsIObserver))
		throw CR.NS_ERROR_NO_INTERFACE;
		return this;
	},

	observe: function(aSubject, aTopic, aData)
	{
		switch(aTopic)
		{
			case "xpcom-startup":
				// this is run very early, right after XPCOM is initialized, but before
				// user profile information is applied. Register ourselves as an observer
				// for 'profile-after-change' and 'quit-application'.
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
				obsSvc.addObserver(this, "profile-after-change", false);
				obsSvc.addObserver(this, "quit-application", false);
				break;

			case "profile-after-change":
				this.removeAutomatic('open');
				break;

			case "quit-application":
				this.removeAutomatic('close');
				break;

			default:
				throw Components.Exception("Unknown topic: " + aTopic);
		}
	},

	removeAutomatic: function(timeEvent)
	{
		// Get the debug level
		// Levels are: 1-All alerts, 5-Failure alerts, 10-No alerts
		this.debugLevel = 10;
		try {this.debugLevel = this.preferences.getIntPref("debugLevel");}catch (err) {}

		var automaticRemove = 'never';
		try
		{
			automaticRemove = this.preferences.getCharPref("automaticRemove");
		}
		catch (err)	{}
		if (timeEvent == automaticRemove)
		{
			// Init the File/Directory objects
			var flashAppDir = new this.nsILocalFile();
			var flashSettingDir = new this.nsILocalFile();
			var flashSettingFile = new this.nsILocalFile();
			var flashObjectDir = new this.nsILocalFile();

			// Test all the flashAppDirXXX to see which one is for this OS
			var flashAppDirPref = this.testFlashDir();

			// If there is a valid path delete files
			if (flashAppDirPref !== null)
			{

				flashAppDir.initWithPath(flashAppDirPref);
				var flashObjectDirSet = false;
				flashObjectDir.initWithFile(flashAppDir);
				flashObjectDir.appendRelativePath('#SharedObjects');
				if (flashObjectDir.exists() && flashObjectDir.isDirectory())
				{
					try
					{
						var entries = flashObjectDir.directoryEntries;
						while(entries.hasMoreElements())
						{
							var entry = entries.getNext();
							entry.QueryInterface(Components.interfaces.nsIFile);
							if (entry.isDirectory())
							{
								flashObjectDir.initWithFile(entry);
								flashObjectDirSet = true;
								break;
							}
						}
					} catch(err) {}
					// Delete settings directories
					if (flashObjectDirSet)
					{
						try
						{
							var entries = flashObjectDir.directoryEntries;
							while(entries.hasMoreElements())
							{
								var entry = entries.getNext();
								entry.QueryInterface(Components.interfaces.nsIFile);
								if (entry.isDirectory())
								{
									try
									{
										entry.remove(true);
										this.jsdump('OBJECTION Remove lso:success:' + entry.path, 2);
									}
									catch (err)
									{
										this.jsdump('OBJECTION Remove lso:failed:' + entry.path + ' err:' + err, 5);
									}
								}
							}
						} catch(err)	{}
					}
					else
					{
						this.jsdump('OBJECTION Could not find:flashObjectDir:' + flashObjectDir.path, 3);
					}
				}
				automaticRemove = false;
				try
				{
					automaticRemoveSite = this.preferences.getBoolPref("automaticRemoveSite");
				}
				catch (err) {}
				// Remove the site settings as well, if requested
				if (automaticRemoveSite)
				{
					// Locate settings directories
					flashSettingDir.initWithFile(flashAppDir);
					flashSettingDir.appendRelativePath('macromedia.com');
					flashSettingDir.appendRelativePath('support');
					flashSettingDir.appendRelativePath('flashplayer');
					flashSettingDir.appendRelativePath('sys');
					if (flashSettingDir.exists() && flashSettingDir.isDirectory())
					{
						this.jsdump('OBJECTION flashSettingDir:' + flashSettingDir.path, 3);
						// Delete all lso directories (including the files)
						try
						{
							var entries = flashSettingDir.directoryEntries;
							while(entries.hasMoreElements())
							{
								var entry = entries.getNext();
								entry.QueryInterface(Components.interfaces.nsIFile);
								if (entry.isDirectory())
								{
									try
									{
										entry.remove(true);
										this.jsdump('OBJECTION Remove setting:success:' + entry.path, 2);
									}
									catch (err)
									{
										this.jsdump('OBJECTION Remove setting:failed:' + entry.path + ' err:' + err, 5);
									}
								}
							}
						} catch(err) {}
					}
				}
			}
		}
//					this.promptService.alert(null,'Objection','so far...');
	},

	testFlashDir: function()
	{
		var tempFlashDirPref = "";
		var tempFlashDir = new this.nsILocalFile();
		if (this.preferences.prefHasUserValue("flashAppDirWin32") && typeof(this.preferences.getCharPref("flashAppDirWin32")) != 'undefined')
		{
			try
			{
				tempFlashDirPref = this.preferences.getCharPref("flashAppDirWin32");
				tempFlashDir.initWithPath(tempFlashDirPref);
				if (tempFlashDir.exists() && tempFlashDir.isDirectory())
				{
					this.jsdump('OBJECTION Platform:Win32:yes', 4);
					return tempFlashDirPref;
				}
			}
			catch(err)
			{
				this.jsdump('OBJECTION Platform:Win32:no', 1);
			}
		}
		if (this.preferences.prefHasUserValue("flashAppDirMacPPC") && typeof(this.preferences.getCharPref("flashAppDirMacPPC")) != 'undefined')
		{
			try
			{
				tempFlashDirPref = this.preferences.getCharPref("flashAppDirMacPPC");
				tempFlashDir.initWithPath(tempFlashDirPref);
				if (tempFlashDir.exists() && tempFlashDir.isDirectory())
				{
					this.jsdump('OBJECTION Platform:MacPPC:yes', 4);
					return tempFlashDirPref;
				}
			}
			catch(err)
			{
				this.jsdump('OBJECTION Platform:MacPPC:no', 1);
			}
		}
		if (this.preferences.prefHasUserValue("flashAppDirOther") && typeof(this.preferences.getCharPref("flashAppDirOther")) != 'undefined')
		{
			try
			{
				tempFlashDirPref = this.preferences.getCharPref("flashAppDirOther");
				tempFlashDir.initWithPath(tempFlashDirPref);
				if (tempFlashDir.exists() && tempFlashDir.isDirectory())
				{
					this.jsdump('OBJECTION Platform:Other:yes', 4);
					return tempFlashDirPref;
				}
			}
			catch(err)
			{
				this.jsdump('OBJECTION Platform:Other:no', 1);
			}
		}
		if (this.preferences.prefHasUserValue("flashAppDir") && typeof(this.preferences.getCharPref("flashAppDir")) != 'undefined')
		{
			try
			{
				tempFlashDirPref = this.preferences.getCharPref("flashAppDir");
				tempFlashDir.initWithPath(tempFlashDirPref);
				if (tempFlashDir.exists() && tempFlashDir.isDirectory())
				{
					this.jsdump('OBJECTION Platform:Old:yes', 4);
					return tempFlashDirPref;
				}
			}
			catch(err)
			{
				this.jsdump('OBJECTION Platform:Old:no', 1);
			}
		}
		return null;
	},

	jsdump: function(str, level)
	{
		if (level >= this.debugLevel)
		{
  			Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).logStringMessage(str);
		}
	}
};

// constructors for objects we want to XPCOMify
var objects = [ObjectionStartup];

/*
 * Registration code.
 *
 */

const CI = Components.interfaces, CC = Components.classes, CR = Components.results;

const MY_OBSERVER_NAME = "ObjectionStartup";

function FactoryHolder(aObj) {
  this.CID        = aObj.prototype.classID;
  this.contractID = aObj.prototype.contractID;
  this.className  = aObj.prototype.classDescription;
  this.factory = {
    createInstance: function(aOuter, aIID) {
      if(aOuter)
        throw CR.NS_ERROR_NO_AGGREGATION;
      return (new this.constructor).QueryInterface(aIID);
    }
  };
  this.factory.constructor = aObj;
}

var gModule = {
  registerSelf: function (aComponentManager, aFileSpec, aLocation, aType)
  {
    aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.registerFactoryLocation(obj.CID, obj.className,
        obj.contractID, aFileSpec, aLocation, aType);
    }

    var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
    catman.addCategoryEntry("xpcom-startup", MY_OBSERVER_NAME,
      ObjectionStartup.prototype.contractID, true, true);
    catman.addCategoryEntry("xpcom-shutdown", MY_OBSERVER_NAME,
      ObjectionStartup.prototype.contractID, true, true);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation) {
    var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
    catman.deleteCategoryEntry("xpcom-startup", MY_OBSERVER_NAME, true);

    aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.unregisterFactoryLocation(obj.CID, aFileSpec);
    }
  },

  getClassObject: function(aComponentManager, aCID, aIID) {
    if (!aIID.equals(CI.nsIFactory)) throw CR.NS_ERROR_NOT_IMPLEMENTED;

    for (var key in this._objects) {
      if (aCID.equals(this._objects[key].CID))
        return this._objects[key].factory;
    }

    throw CR.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aComponentManager) {
    return true;
  },

  _objects: {} //FactoryHolder
};

function NSGetModule(compMgr, fileSpec)
{
  for(var i in objects)
    gModule._objects[i] = new FactoryHolder(objects[i]);
  return gModule;
}